{-# LANGUAGE DeriveGeneric #-}
{-# LANGUAGE DuplicateRecordFields #-}

module Types.WebSocketMessages.WebSocketMessages
  ( WebSocketMessage (..),
    SetClientInfo (..),
    JoinRoom (..),
    AllChatMessageIncoming (..),
    AllChatMessageOutgoing (..),
  )
where

import ClassyPrelude
import Data.Aeson
  ( FromJSON (parseJSON),
    Options (sumEncoding),
    SumEncoding (..),
    ToJSON,
    defaultOptions,
    genericParseJSON,
  )
import Types.User (User)

data WebSocketMessage = ClientInfoMessage SetClientInfo | JoinRoomMessage JoinRoom | AllChatMessageIncomingMessage AllChatMessageIncoming
  deriving (Generic)

instance FromJSON WebSocketMessage where
  parseJSON = genericParseJSON defaultOptions {sumEncoding = UntaggedValue}

data SetClientInfo = SetClientInfo
  { displayName :: Text
  }
  deriving (Generic, Show)

instance FromJSON SetClientInfo

data JoinRoom = JoinRoom
  { roomName :: Text
  }
  deriving (Generic, Show)

instance FromJSON JoinRoom

data AllChatMessageIncoming = AllChatMessageIncoming
  { content :: Text
  }
  deriving (Generic, Show)

instance FromJSON AllChatMessageIncoming

data AllChatMessageOutgoing = AllChatMessageOutgoing
  { content :: Text,
    sender :: User
  }
  deriving (Generic, Show)

instance ToJSON AllChatMessageOutgoing
